# passctrl.tcl for netbots.tcl v3.20
# designed to work with eggdrop 1.3.28 or higher
# Johoho's Eggdrop Page - http://johoho.tuts.net/eggdrop/
# passctrl.tcl copyright (c) 1999,2000 by johoho <johoho@hojo-net.de>
# netbots.tcl copyright (c) 1998-2000 by slennox <slennox@egghelp.org>

## passctrl.tcl component script v1.10.0, 07.05.2000 ##

# History:
# 
# v0.10.0, 02.05.2000 - initial design and concept
# v0.30.0, 03.05.2000 - fixed bugs
# v0.50.0, 04.05.2000 - changed logging, handling, etc.
# v1.00.0, 06.05.2000 - public release
# v1.10.0, 07.05.2000 - added dictionary support

# thanks to NML_375 for his regexp help (crappy brackets :)! thanks slennox for netbots.tcl
# credits to Abraham for his bug and feature reports

proc pc_msgpasschk  {nick uhost hand arg} {
  set pass [lindex $arg 0]
  set isok "[passchk $hand $pass $arg]"
  if {$isok == 1} {
    puthelp "PRIVMSG $nick :Your password is active now"
    setuser $hand pass $pass
    putlog "($uhost) !$hand! PASS..."
  } else {
    puthelp "PRIVMSG $nick :$isok"
    return 0
  }
}

proc pc_dccnewpasschk {hand idx arg} {
  set pass [lindex $args 0]
  if {$pass == ""} {
    putdcc $idx "Usage: newpass <newpassword>"
    return 0    
  }
  if {[passwdok $hand $pass]} {
    putdcc $idx "There's not much sense changing your passwort to the same"
    return 0
  }
  set isok "[passchk $hand $pass $arg]"
  if {$isok == 1} {
    putdcc $idx "Your new password is active now"
    setuser $hand pass $pass
    putcmdlog "#$hand# newpass..."
  } else {
    putdcc $idx "$isok"
    return 0
  }
}

proc pc_dccchpasschk {hand idx arg} {
global pc_owner
  set whom [lindex $arg 0]
  set pass [lindex $arg 1]
  if {$whom == ""} {
    putdcc $idx "Usage: chpass <handle> \[password\]"
    return 0    
  }
  set isok "[passchk $whom $pass $arg]"
  if {($isok == 1) || ($pass == "") || ($pc_owner == 1)} {
    putdcc $idx "$whom's password is changed now"
    setuser $whom pass $pass
    return 1
  } else {
    putdcc $idx "$isok"
    return 0
  }
}

proc passchk {hand pass args} {
global pc_wordlist pc_length pc_strength 
  if {$pc_strength > 32} { putlog "invalid pc_strength setting, check the readme" ; return 0 }
  if {[string length $pass] < $pc_length} {
    return "The password must be at least $pc_length chars long."
  }
  set tmp $pc_strength
  if {$tmp >= 16} {
    if {![file exists $pc_wordlist]} {
      putlog "ERROR: there was no wordlist file called '$pc_wordlist' found, password check aborted."
      return "There is a problem with the internal verification procedures. Process canceled."
    }
    set f [open $sc_file r]
    while {![eof $f]} {
      set line [gets $f]
      if {[string match *$pass* $line]} {
        set isin 1
        break
      }
    }
    close $f
    if {[info exists isin]} {
      return "The password you have chosen ist not allowed"
    } else {
      incr tmp -16
    }
  }
  if {$tmp >= 8} {
    if {![regexp -- {([a-z][A-Z]|[A-Z][a-z])} $pass]} {
      return "The password must contain at least one lower and upper case character"
    } else {
      incr tmp -8
    }
  }
  if {$tmp >= 4} {
    if {![regexp -- {[^a-zA-z0-9]} $pass]} {
      return "The password must contain at least one special character"
    } else {
      incr tmp -4
    }
  }
  if {$tmp >= 2} {
    if {![regexp -- {[0-9]} $pass]} {
      return "The password must contain at least one numeric character"
    } else {
      incr tmp -2
    }
  }
  if {$tmp >= 1} {
    if {![regexp -nocase -- {[a-z]} $pass]} {
      return "The password must contain at least one normal character"
    } else {
      incr tmp -1
    }
  }
  if {[string match *[string tolower $pass]* [string tolower $hand]]} {
    return "The Password is too similar to the handle $hand."
  }
  for {set i [string length $pass]} {$i > 0} {} {
    append reverse [string index $pass [incr i -1]]
  }
  if {[string match *$reverse* *$hand*]} {
    return "The Password is too similar to the reversed version of the handle $hand ($reverse)."
  }
  if {$tmp >= 0} {
    return 1
  }
}

unbind msg -|- pass *msg:pass
unbind dcc - newpass *dcc:newpass
unbind dcc - chpass *dcc:chpass
bind msg -|- pass pc_msgpasschk 
bind dcc -|- newpass pc_dccnewpasschk
bind dcc m|m chpass pc_dccchpasschk
